from typing import (
    Any,
    Dict,
    List,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="RequestedAttributes")


@_attrs_define
class RequestedAttributes:
    """
    Attributes:
        attributes (List[str]):
        holder_identifier (str):
        holder_type (str):
    """

    attributes: List[str]
    holder_identifier: str
    holder_type: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        attributes = self.attributes

        holder_identifier = self.holder_identifier

        holder_type = self.holder_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "attributes": attributes,
                "holderIdentifier": holder_identifier,
                "holderType": holder_type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        attributes = cast(List[str], d.pop("attributes"))

        holder_identifier = d.pop("holderIdentifier")

        holder_type = d.pop("holderType")

        requested_attributes = cls(
            attributes=attributes,
            holder_identifier=holder_identifier,
            holder_type=holder_type,
        )

        requested_attributes.additional_properties = d
        return requested_attributes

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
